@@ -88,7 +88,7 @@ class Menu(object):
     :param rows: Number of rows of each column, None if there's only 1 column
     :type rows: int, None
     :param theme: Menu theme object, if None use the default theme
-    :type theme: :py:class:`pygame_menu.themes.Theme`, None
+    :type theme: :py:class:`pygame_menu.themes.Theme`
     """
 
     def __init__(self,
@@ -126,7 +126,7 @@ class Menu(object):
         assert isinstance(mouse_motion_selection, bool)
         assert isinstance(mouse_visible, bool)
         assert isinstance(rows, (int, type(None)))
-        assert isinstance(theme, _themes.Theme), 'theme bust be an Theme object instance'
+        assert isinstance(theme, _themes.Theme), 'theme bust be an pygame_menu.themes.Theme object instance'
         assert isinstance(title, str)
 
         # Assert theme
@@ -242,40 +242,52 @@ class Menu(object):
         self._mouse_visible_default = mouse_visible
 
         # Create Menu bar (title)
-        self._menubar = _widgets.MenuBar(label=title,
-                                         width=self._width,
-                                         back_box=back_box,
-                                         mode=self._theme.title_bar_style,
-                                         onreturn=self._back,
-                                         background_color=self._theme.title_background_color)
+        self._menubar = _widgets.MenuBar(
+            title=title,
+            width=self._width,
+            back_box=back_box,
+            mode=self._theme.title_bar_style,
+            onreturn=self._back,
+            background_color=self._theme.title_background_color
+        )
         self._menubar.set_menu(self)
-        self._menubar.set_title(title=title,
-                                offsetx=theme.title_offset[0],
-                                offsety=theme.title_offset[1])
-        self._menubar.set_font(font=self._theme.title_font,
-                               font_size=self._theme.title_font_size,
-                               color=self._theme.title_font_color,
-                               selected_color=self._theme.title_font_color)
-        self._menubar.set_shadow(enabled=self._theme.title_shadow,
-                                 color=self._theme.title_shadow_color,
-                                 position=self._theme.title_shadow_position,
-                                 offset=self._theme.title_shadow_offset)
+        self._menubar.set_title(
+            title=title,
+            offsetx=theme.title_offset[0],
+            offsety=theme.title_offset[1]
+        )
+        self._menubar.set_font(
+            font=self._theme.title_font,
+            font_size=self._theme.title_font_size,
+            color=self._theme.title_font_color,
+            selected_color=self._theme.title_font_color,
+            background_color=None,
+            antialias=self._theme.title_font_antialias
+        )
+        self._menubar.set_shadow(
+            enabled=self._theme.title_shadow,
+            color=self._theme.title_shadow_color,
+            position=self._theme.title_shadow_position,
+            offset=self._theme.title_shadow_offset
+        )
         self._menubar.set_controls(self._joystick, self._mouse)
 
         # Scrolling area
         self._widgets_surface = None
-        self._scroll = ScrollArea(area_width=self._width,
-                                  area_color=self._theme.background_color,
-                                  area_height=self._height - self._menubar.get_rect().height,
-                                  extend_y=self._menubar.get_rect().height,
-                                  scrollbar_color=self._theme.scrollbar_color,
-                                  scrollbar_slider_color=self._theme.scrollbar_slider_color,
-                                  scrollbar_slider_pad=self._theme.scrollbar_slider_pad,
-                                  scrollbar_thick=self._theme.scrollbar_thick,
-                                  shadow=self._theme.scrollbar_shadow,
-                                  shadow_color=self._theme.scrollbar_shadow_color,
-                                  shadow_offset=self._theme.scrollbar_shadow_offset,
-                                  shadow_position=self._theme.scrollbar_shadow_position)
+        self._scroll = ScrollArea(
+            area_width=self._width,
+            area_color=self._theme.background_color,
+            area_height=self._height - self._menubar.get_rect().height,
+            extend_y=self._menubar.get_rect().height,
+            scrollbar_color=self._theme.scrollbar_color,
+            scrollbar_slider_color=self._theme.scrollbar_slider_color,
+            scrollbar_slider_pad=self._theme.scrollbar_slider_pad,
+            scrollbar_thick=self._theme.scrollbar_thick,
+            shadow=self._theme.scrollbar_shadow,
+            shadow_color=self._theme.scrollbar_shadow_color,
+            shadow_offset=self._theme.scrollbar_shadow_offset,
+            shadow_position=self._theme.scrollbar_shadow_position
+        )
 
     def get_current(self):
         """
@@ -362,6 +374,7 @@ class Menu(object):
         # Configure and add the button
         self._configure_widget(widget=widget, **attributes)
         self._append_widget(widget)
+
         return widget
 
     def add_color_input(self,
@@ -434,15 +447,17 @@ class Menu(object):
             cursor_color=self._theme.cursor_color,
             input_separator=input_separator,
             input_underline=input_underline,
-            label=title,
             onchange=onchange,
             onreturn=onreturn,
             prev_size=previsualization_width,
+            title=title,
             **kwargs
         )
+
         self._configure_widget(widget=widget, **attributes)
         widget.set_value(default)
         self._append_widget(widget)
+
         return widget
 
     def add_image(self,
@@ -493,6 +508,7 @@ class Menu(object):
             scale=scale,
             scale_smooth=scale_smooth,
         )
+
         widget.is_selectable = selectable
         self._configure_widget(widget=widget, **attributes)
         self._append_widget(widget)
@@ -543,14 +559,14 @@ class Menu(object):
         # Wrap text to menu width (imply additional calls to render functions)
         if max_char < 0:
             dummy_attrs = self._filter_widget_attributes(kwargs.copy())
-            dummy = _widgets.Label(label=title)
+            dummy = _widgets.Label(title=title)
             self._configure_widget(dummy, **dummy_attrs)
             max_char = int(1.0 * self._width * len(title) / dummy.get_rect().width)
 
         # If no overflow
         if len(title) <= max_char or max_char == 0:
             attributes = self._filter_widget_attributes(kwargs)
-            widget = _widgets.Label(label=title, label_id=label_id)
+            widget = _widgets.Label(title=title, label_id=label_id)
             widget.is_selectable = selectable
             self._configure_widget(widget=widget, **attributes)
             self._append_widget(widget)
@@ -558,11 +574,15 @@ class Menu(object):
             self._check_id_duplicated(label_id)  # Before adding + LEN
             widget = []
             for line in textwrap.wrap(title, max_char):
-                widget.append(self.add_label(title=line,
-                                             label_id=label_id + '+' + str(len(widget) + 1),
-                                             max_char=max_char,
-                                             selectable=selectable,
-                                             **kwargs))
+                widget.append(
+                    self.add_label(
+                        title=line,
+                        label_id=label_id + '+' + str(len(widget) + 1),
+                        max_char=max_char,
+                        selectable=selectable,
+                        **kwargs
+                    )
+                )
         return widget
 
     def add_selector(self,
@@ -626,14 +646,16 @@ class Menu(object):
         widget = _widgets.Selector(
             default=default,
             elements=items,
-            label=title,
             onchange=onchange,
             onreturn=onreturn,
             selector_id=selector_id,
+            title=title,
             **kwargs
         )
+
         self._configure_widget(widget=widget, **attributes)
         self._append_widget(widget)
+
         return widget
 
     def add_text_input(self,
@@ -727,7 +749,6 @@ class Menu(object):
             cursor_selection_enable=cursor_selection_enable,
             input_type=input_type,
             input_underline=input_underline,
-            label=title,
             maxchar=maxchar,
             maxwidth=maxwidth,
             onchange=onchange,
@@ -735,12 +756,15 @@ class Menu(object):
             password=password,
             tab_size=tab_size,
             textinput_id=textinput_id,
+            title=title,
             valid_chars=valid_chars,
             **kwargs
         )
+
         self._configure_widget(widget=widget, **attributes)
         widget.set_value(default)
         self._append_widget(widget)
+
         return widget
 
     def add_vertical_margin(self, margin):
@@ -752,10 +776,16 @@ class Menu(object):
         :return: Widget object
         :rtype: :py:class:`pygame_menu.widgets.VMargin`
         """
+        assert isinstance(margin, (int, float))
+
+        # Filter widget attributes to avoid passing them to the callbacks
         attributes = self._filter_widget_attributes({'margin': (0, margin)})
+
         widget = _widgets.VMargin()
+
         self._configure_widget(widget=widget, **attributes)
         self._append_widget(widget)
+
         return widget
 
     def _filter_widget_attributes(self, kwargs):
@@ -764,9 +794,9 @@ class Menu(object):
         The valid (key, value) are removed from the initial
         dictionary.
 
-        :param kwargs: input attributes
+        :param kwargs: Input attributes
         :type kwargs: dict
-        :return: dictionary of valid  attributes
+        :return: Dictionary of valid attributes
         :rtype: dict
         """
         attributes = {}
@@ -786,6 +816,14 @@ class Menu(object):
         _utils.assert_vector2(background_inflate)
         attributes['background_inflate'] = background_inflate
 
+        attributes['font_antialias'] = self._theme.widget_font_antialias
+
+        font_background_color = None
+        if self._theme.widget_font_background_color_from_menu:
+            if isinstance(self._theme.background_color, tuple):
+                font_background_color = self._theme.background_color
+        attributes['font_background_color'] = font_background_color
+
         font_color = kwargs.pop('font_color', self._theme.widget_font_color)
         _utils.assert_color(font_color)
         attributes['font_color'] = font_color
@@ -856,19 +894,27 @@ class Menu(object):
         :type kwargs: any
         :return: None
         """
+        assert isinstance(widget, _widgets.core.Widget)
+        assert widget.get_menu() is None, 'widget cannot have an instance of menu'
         _col = int((len(self._widgets) - 1) // self._rows)  # Column position
         widget.set_menu(self)
         self._check_id_duplicated(widget.get_id())
-        widget.set_font(font=kwargs['font_name'],
-                        font_size=kwargs['font_size'],
-                        color=kwargs['font_color'],
-                        selected_color=kwargs['selection_color'])
+        widget.set_font(
+            font=kwargs['font_name'],
+            font_size=kwargs['font_size'],
+            color=kwargs['font_color'],
+            selected_color=kwargs['selection_color'],
+            background_color=kwargs['font_background_color'],
+            antialias=kwargs['font_antialias']
+        )
         if self._force_fit_text and self._column_max_width[_col] is not None:
             widget.set_max_width(self._column_max_width[_col] - kwargs['selection_effect'].get_width())
-        widget.set_shadow(enabled=kwargs['shadow'],
-                          color=kwargs['shadow_color'],
-                          position=kwargs['shadow_position'],
-                          offset=kwargs['shadow_offset'])
+        widget.set_shadow(
+            enabled=kwargs['shadow'],
+            color=kwargs['shadow_color'],
+            position=kwargs['shadow_position'],
+            offset=kwargs['shadow_offset']
+        )
         widget.set_controls(self._joystick, self._mouse)
         widget.set_alignment(kwargs['align'])
         widget.set_margin(*kwargs['margin'])
@@ -877,12 +923,13 @@ class Menu(object):
 
     def _append_widget(self, widget):
         """
-        Add a widget to the list.
+        Add a widget to the list of widgets.
 
         :param widget: Widget object
         :type widget: :py:class:`pygame_menu.widgets.core.widget.Widget`
         """
         assert isinstance(widget, _widgets.core.Widget)
+        assert widget.get_menu() == self, 'widget cannot have a different instance of menu'
         if self._columns > 1:
             max_elements = self._columns * self._rows
             assert len(self._widgets) + 1 <= max_elements, \
@@ -903,6 +950,7 @@ class Menu(object):
         :type widget: :py:class:`pygame_menu.widgets.core.widget.Widget`
         :return: None
         """
+        assert widget is not None, 'widget cannot be None'
         assert isinstance(widget, _widgets.core.Widget)
         try:
             indx = self._widgets.index(widget)  # If not exists this raises ValueError
@@ -1110,6 +1158,7 @@ class Menu(object):
         :type widget_id: str
         :return: None
         """
+        assert isinstance(widget_id, str)
         for widget in self._widgets:  # type: _widgets.core.Widget
             if widget.get_id() == widget_id:
                 raise ValueError('widget ID="{0}" is duplicated'.format(widget_id))
@@ -1225,7 +1274,7 @@ class Menu(object):
         self._widget_offset[1] = self._height * new_pos
         self._current._widgets_surface = None  # Rebuild on the next draw
 
-    def draw(self, surface):
+    def draw(self, surface, clear_surface=False):
         """
         Draw the current Menu into the given surface.
 
@@ -1233,8 +1282,13 @@ class Menu(object):
 
         :param surface: Pygame surface to draw the Menu
         :type surface: :py:class:`pygame.Surface`
+        :param clear_surface: Clear surface using theme default color
+        :type clear_surface: bool
         :return: None
         """
+        assert isinstance(surface, pygame.Surface)
+        assert isinstance(clear_surface, bool)
+
         if not self.is_enabled():
             raise RuntimeError('menu is not enabled, it cannot be drawn')
 
@@ -1243,7 +1297,11 @@ class Menu(object):
         if not self._current._widgets_surface:
             self._current._build_widget_surface()
 
-        # Fill the surface with background function (set from mainloop)
+        # Clear surface
+        if clear_surface:
+            surface.fill(self._theme.surface_clear_color)
+
+        # Call background function (set from mainloop)
         if self._top._background_function is not None:
             self._top._background_function()
 
@@ -1274,6 +1332,9 @@ class Menu(object):
         :type widget: :py:class:`pygame_menu.widgets.core.widget.Widget`, None
         :return: None
         """
+        assert isinstance(surface, pygame.Surface)
+        assert isinstance(widget, (_widgets.core.Widget, type(None)))
+
         if widget is None or not widget.active or not self._mouse_motion_selection:
             return
         window_width, window_height = pygame.display.get_surface().get_size()
@@ -1408,9 +1469,6 @@ class Menu(object):
         # Update mouse
         pygame.mouse.set_visible(self._current._mouse_visible)
 
-        # Surface needs an update
-        menu_surface_needs_update = False
-
         selected_widget = None  # type: _widgets.core.Widget
         if len(self._current._widgets) >= 1:
             selected_widget = self._current._widgets[self._current._index % len(self._current._widgets)]
@@ -1542,16 +1600,15 @@ class Menu(object):
                         new_event.dict['origin'] = self._current._scroll.to_real_position((0, 0))
                         new_event.pos = self._current._scroll.to_world_position(event.pos)
                         selected_widget.update((new_event,))  # This widget can change the current Menu to a submenu
-                        menu_surface_needs_update = menu_surface_needs_update or selected_widget.surface_needs_update()
                         updated = True  # It is updated
                         break
 
-        # Check if the position has changed
+        # Check if the menu widgets size changed, if True, updates the surface
+        # forcing the rendering of all widgets
+        menu_surface_needs_update = False
         if len(self._current._widgets) > 0:
-            menu_surface_needs_update = menu_surface_needs_update or self._current._widgets[
-                self._current._index].surface_needs_update()
-
-        # This forces the rendering of all widgets
+            for widget in self._current._widgets:  # type: _widgets.core.Widget
+                menu_surface_needs_update = menu_surface_needs_update or widget.surface_needs_update()
         if menu_surface_needs_update:
             self._current._widgets_surface = None
 
@@ -1603,7 +1660,7 @@ class Menu(object):
         while True:
             self._current._clock.tick(fps_limit)
 
-            self.draw(surface=surface)
+            self.draw(surface=surface, clear_surface=True)
 
             # If loop, gather events by Menu and draw the background function, if this method
             # returns true then the mainloop will break
@@ -1691,7 +1748,7 @@ class Menu(object):
         :type recursive: bool
         :return: None
         """
-        assert isinstance(sound, (type(self._sounds), type(None)))
+        assert isinstance(sound, (type(self._sounds), type(None))), 'sound must be pygame_menu.Sound type or None'
         if sound is None:
             sound = Sound()
         self._sounds = sound
@@ -1802,11 +1859,17 @@ class Menu(object):
                 dwidget = 1
 
         # Limit the index to the length
-        new_index %= len(current._widgets)
+        total_current = len(current._widgets)
+        new_index %= total_current
         if new_index == current._index:  # Index has not changed
             return
 
         # Get both widgets
+        if current._index >= total_current:  # The length of the menu changed during execution time
+            for i in range(total_current):  # Unselect all posible candidates
+                current._widgets[i].set_selected(False)
+            current._index = 0
+
         old_widget = current._widgets[current._index]  # type: _widgets.core.Widget
         new_widget = current._widgets[new_index]  # type:_widgets.core.Widget
 
@@ -1858,6 +1921,8 @@ class Menu(object):
         :return: Widget object
         :rtype: :py:class:`pygame_menu.widgets.core.widget.Widget`
         """
+        assert isinstance(widget_id, str)
+        assert isinstance(recursive, bool)
         for widget in self._widgets:  # type: _widgets.core.Widget
             if widget.get_id() == widget_id:
                 return widget
