@@ -39,12 +39,15 @@ from pygame_menu.sound import Sound
 from pygame_menu.utils import make_surface, assert_alignment, assert_color, assert_position, assert_vector2
 
 from uuid import uuid4
+import time
 
 
 class Widget(object):
     """
     Widget abstract class.
 
+    :param title: Widget title
+    :type title: str
     :param widget_id: Widget identifier
     :type widget_id: str
     :param onchange: Callback when changing the selector
@@ -56,12 +59,14 @@ class Widget(object):
     """
 
     def __init__(self,
+                 title='',
                  widget_id='',
                  onchange=None,
                  onreturn=None,
                  args=None,
                  kwargs=None
                  ):
+        assert isinstance(title, str)
         assert isinstance(widget_id, str)
         if onchange:
             assert callable(onchange), 'onchange must be callable or None'
@@ -74,11 +79,14 @@ class Widget(object):
         self._alignment = _locals.ALIGN_CENTER
         self._background_color = None
         self._background_inflate = (0, 0)
+        self._events = []  # type: list
         self._id = str(widget_id)
-        self._selected_rect = None  # type: (pygame.rect.Rect,None)
-        self._rect = pygame.Rect(0.0, 0.0, 0.0, 0.0)  # type: (pygame.Rect,None)
         self._margin = (0.0, 0.0)  # type: tuple
         self._max_width = None  # type: (int,float)
+        self._rect = pygame.Rect(0.0, 0.0, 0.0, 0.0)  # type: (pygame.Rect,None)
+        self._selected_rect = None  # type: (pygame.rect.Rect,None)
+        self._selection_time = 0  # type: float
+        self._title = title
 
         self._args = args or []  # type: list
         self._kwargs = kwargs or {}  # type: dict
@@ -97,11 +105,12 @@ class Widget(object):
 
         # Modified in set_font() method
         self._font = None  # type: (pygame.font.Font,None)
-        self._font_name = ''  # type: str
-        self._font_size = 0  # type: int
-        self._font_color = (0, 0, 0)  # type: tuple
-        self._font_selected_color = (255, 255, 255)  # type: tuple
         self._font_antialias = True  # type: bool
+        self._font_background_color = None  # type: (tuple, None)
+        self._font_color = (0, 0, 0)  # type: tuple
+        self._font_name = ''  # type: str
+        self._font_selected_color = (255, 255, 255)  # type: tuple
+        self._font_size = 0  # type: int
 
         # Text shadow
         self._shadow = False  # type: bool
@@ -117,7 +126,7 @@ class Widget(object):
         # then the widget should render and update the hash
         self._last_render_hash = 0  # type: int
 
-        # Stores the last render surface size, updated by
+        # Stores the last render surface size, updated by _check_render_size_changed()
         self._last_render_surface_size = (0, 0)
 
         self._selection_effect = None  # type: Selection
@@ -159,6 +168,29 @@ class Widget(object):
             return True
         return False
 
+    def set_title(self, title):  # lgtm [py/inheritance/incorrect-overridden-signature]
+        """
+        Update the widget title.
+
+        :param title: New title
+        :type title: str
+        :return: None
+        """
+        assert isinstance(title, str)
+        self._title = title
+        self._apply_font()
+        self._render()
+        self._check_render_size_changed()
+
+    def get_title(self):
+        """
+        Return the widget title.
+
+        :return: Widget title
+        :rtype: str
+        """
+        return self._title
+
     def set_background_color(self, color, inflate=(0, 0)):
         """
         Set widget background color.
@@ -367,7 +399,7 @@ class Widget(object):
         """
         raise NotImplementedError('override is mandatory')
 
-    def font_render_string(self, text, color=(0, 0, 0)):
+    def _font_render_string(self, text, color=(0, 0, 0), use_background_color=True):
         """
         Render text.
 
@@ -375,16 +407,33 @@ class Widget(object):
         :type text: str
         :param color: Text color
         :type color: tuple
+        :param use_background_color: Use default background color
+        :type use_background_color: bool
         :return: Text surface
         :rtype: :py:class:`pygame.Surface`
         """
+        assert isinstance(text, str)
         assert isinstance(color, tuple)
-        return self._font.render(text, self._font_antialias, color)
+        assert isinstance(use_background_color, bool)
+        bgcolor = self._font_background_color
+
+        # Background color must be opaque, otherwise the results are quite bad
+        if isinstance(bgcolor, (tuple, list)) and len(bgcolor) == 4 and bgcolor[3] != 255:
+            bgcolor = None
+
+        # Disable
+        if not use_background_color:
+            bgcolor = None
+
+        return self._font.render(text, self._font_antialias, color, bgcolor)
 
     def _check_render_size_changed(self):
         """
         Check the size changed after rendering.
-        This method should be used only on widgets that can change in size.
+        This method should be used only on widgets that can change in size, or if the size
+        is changed during execution time (like set_title).
+        The update status (needs update if render size changed) is returned by
+        Widget.surface_needs_update() method.
 
         :return: Boolean, if True the size changed
         :rtype: bool
@@ -406,7 +455,7 @@ class Widget(object):
         :return: Text surface
         :rtype: :py:class:`pygame.Surface`
         """
-        text = self.font_render_string(string, color)
+        text = self._font_render_string(string, color)
 
         # Create surface
         surface = make_surface(width=text.get_width(),
@@ -415,7 +464,7 @@ class Widget(object):
 
         # Draw shadow first
         if self._shadow:
-            text_bg = self._font.render(string, self._font_antialias, self._shadow_color)
+            text_bg = self._font_render_string(string, self._shadow_color)
             surface.blit(text_bg, self._shadow_tuple)
 
         surface.blit(text, (0, 0))
@@ -441,7 +490,7 @@ class Widget(object):
             return True
         return False
 
-    def set_font(self, font, font_size, color, selected_color, antialias=True):
+    def set_font(self, font, font_size, color, selected_color, background_color, antialias=True):
         """
         Set the text font.
 
@@ -453,6 +502,8 @@ class Widget(object):
         :type color: tuple
         :param selected_color: Text color when widget is selected
         :type selected_color: tuple
+        :param background_color: Font background color
+        :type background_color: tuple
         :param antialias: Determines if antialias is applied to font (uses more processing power)
         :type antialias: bool
         :return: None
@@ -461,13 +512,17 @@ class Widget(object):
         assert isinstance(font_size, int)
         assert isinstance(color, tuple)
         assert isinstance(selected_color, tuple)
+        assert isinstance(background_color, (tuple, type(None)))
         assert isinstance(antialias, bool)
-        self._font_name = font
+
         self._font = _fonts.get_font(font, font_size)
-        self._font_size = font_size
-        self._font_color = color
-        self._font_selected_color = selected_color
         self._font_antialias = antialias
+        self._font_background_color = background_color
+        self._font_color = color
+        self._font_name = font
+        self._font_selected_color = selected_color
+        self._font_size = font_size
+
         self._apply_font()
 
     def get_font_info(self):
@@ -557,10 +612,24 @@ class Widget(object):
         self.active = False
         if selected:
             self._focus()
+            self._selection_time = time.time()
         else:
             self._blur()
+            self._events = []  # Remove events
         self._render()
 
+    def get_selected_time(self):
+        """
+        Return time the widget has been selected in miliseconds.
+        If the widget is not currently selected, return 0.
+
+        :return: Time in ms
+        :rtype: float
+        """
+        if not self.selected:
+            return 0
+        return (time.time() - self._selection_time) * 1000
+
     def _focus(self):
         """
         Function that is executed when the widget receives a focus (is selected).
@@ -686,3 +755,31 @@ class Widget(object):
         :rtype: bool
         """
         raise NotImplementedError('override is mandatory')
+
+    def _add_event(self, event):
+        """
+        Add a custom event to the widget for the next update().
+
+        :param event: Custom event
+        :type event: :py:class:`pygame.event.Event`
+        """
+        self._events.append(event)
+
+    def _merge_events(self, events):
+        """
+        Append widget events to events list.
+
+        :param events: Event list
+        :type events: list[:py:class:`pygame.event.Event`]
+        :return: Augmented event list
+        :rtype: list[:py:class:`pygame.event.Event`]
+        """
+        if len(self._events) == 0:
+            return events
+        copy_events = []
+        for e in events:
+            copy_events.append(e)
+        for e in self._events:
+            copy_events.append(e)
+        self._events = []
+        return copy_events
